@@ -5,6 +5,7 @@ Inspired by https://github.com/Hargne/jest-html-reporter
 
 from __future__ import absolute_import
 
+import atexit
 import base64
 import os
 import time
@@ -168,98 +169,102 @@ class Feature:
         """
         Converts this object to HTML.
         """
-        # Feature Panel.
-        with div(cls="feature-panel"):
-            with div(cls="feature-title"):
-                # Generate icon if present.
-                if self.icon:
-                    with div(cls="feature-icon"):
-                        img(src=self.icon)
 
-                # Generate content of the panel.
-                if self.high_contrast_button:
-                    # Making sure there is a functioning button.
-                    span(f"Feature: {self.name}")
-                    with a(
-                        onclick="toggle_contrast('embed')",
-                        href="#/",
-                        cls="flex-left-space",
-                    ):
-                        # Creating the actual text content which is clickable.
-                        span("[High contrast toggle]", cls="button-toggle")
+        # Feature Title.
+        with div(cls="feature-title flex-gap"):
+            # Generate icon if present.
+            if self.icon:
+                with div(cls="feature-icon"):
+                    img(src=self.icon)
 
-                    # After the High Contrast make a Summary toggle button.
-                    with a(
-                        onclick="collapsible_summary('feature-summary-container')",
-                        href="#/",
-                    ):
-                        # Creating the actual text content which is clickable.
-                        span("[Summary]", cls="button-toggle")
+            # Generate content of the feature title bar.
+            span(f"Feature: {self.name}")
 
-                # On following features do not generate the buttons.
-                else:
-                    span(f"Feature: {self.name}")
+            if self.high_contrast_button:
+                # Making sure there is a functioning button.
+                # Creating High Contrast oggle which is clickable.
+                span(
+                    "High contrast toggle",
+                    cls="button flex-left-space",
+                    onclick="toggle_contrast('embed')",
+                )
 
-            # Generate summary.
-            summary_display = "display: none"
-            if formatter.show_summary:
-                summary_display = ""
-            with div(
-                cls="feature-summary-container",
-                id=f"f{self.counter}",
-                style=summary_display,
-            ):
-                # Generating feature commentary.
-                flex_left_space = "flex-left-space" if self.description else ""
+                # Creating Summary which is clickable.
+                span(
+                    "Summary",
+                    cls="button",
+                    onclick="collapsible_summary('feature-summary-container')",
+                )
 
-                # with div(cls=""):
-                if self.description:
-                    pre(
-                        self.description,
-                        cls="feature-summary-commentary",
-                    )
+        # Generate summary.
+        summary_display = "display: none"
+        if formatter.show_summary:
+            summary_display = ""
+        with div(
+            cls="feature-summary-container flex-gap",
+            id=f"f{self.counter}",
+            style=summary_display,
+        ):
+            # Generating feature commentary.
+            flex_left_space = "flex-left-space" if self.description else ""
 
-                # Generating Summary results.
-                with div(cls=f"feature-summary-stats {flex_left_space}"):
-                    stats = self.get_feature_stats()
+            # with div(cls=""):
+            if self.description:
+                pre(
+                    self.description,
+                    cls="feature-summary-commentary",
+                )
 
-                    for stat, value in stats.items():
-                        div(
-                            f"{stat}: {value}",
-                            cls=f"feature-summary-row {stat.lower()}",
-                        )
+            # Generating Summary results.
+            with div(cls=f"feature-summary-stats {flex_left_space}"):
+                stats = self.get_feature_stats()
 
-                with div(cls="feature-summary-stats flex-left-space"):
+                for stat, value in stats.items():
                     div(
-                        f"Started: {self.start_time.strftime(formatter.date_format)}",
-                        cls="feature-summary-row",
+                        f"{stat}: {value}",
+                        cls=f"feature-summary-row {stat.lower()}",
                     )
-                    duration = (self.finish_time - self.start_time).total_seconds()
-                    div(f"Duration: {duration:.2f}", cls="feature-summary-row")
-                    div(
-                        f"Finished: {self.finish_time.strftime(formatter.date_format)}",
-                        cls="feature-summary-row",
-                    )
-                # Generating clickable buttons for collapsing/expanding.
-                with div(cls="feature-summary-stats"):
-                    with a(onclick="expander('expand_all', this)", href="#"):
-                        div("[Expand All]", cls="feature-summary-row button")
-                    with a(onclick="expander('collapse_all', this)", href="#"):
-                        div("[Collapse All]", cls="feature-summary-row button")
-                    with a(onclick="expander('expand_all_failed', this)", href="#"):
-                        div("[Expand All Failed]", cls="feature-summary-row button")
 
-                if formatter.additional_info:
-                    with div(
-                        cls="feature-additional-info-container", id="additional-info"
-                    ):
-                        # Generating Additional info results
-                        with div(cls="feature-additional-info"):
-                            for key, item in formatter.additional_info.items():
-                                div(
-                                    f"{key}: {item}",
-                                    cls=f"feature-additional-info-row {key.lower()}",
-                                )
+            # Generating Started/Duration/Finished message.
+            with div(cls="feature-summary-stats flex-left-space"):
+                div(
+                    f"Started: {self.start_time.strftime(formatter.date_format)}",
+                    cls="feature-summary-row",
+                )
+                duration = (self.finish_time - self.start_time).total_seconds()
+                div(f"Duration: {duration:.2f}", cls="feature-summary-row")
+                div(
+                    f"Finished: {self.finish_time.strftime(formatter.date_format)}",
+                    cls="feature-summary-row",
+                )
+
+            # Generating clickable buttons for collapsing/expanding.
+            with div(cls="feature-summary-stats"):
+                span(
+                    "Expand All",
+                    cls="button display-block",
+                    onclick="expander('expand_all', this)",
+                )
+                span(
+                    "Collapse All",
+                    cls="button display-block",
+                    onclick="expander('collapse_all', this)",
+                )
+                span(
+                    "Expand All Failed",
+                    cls="button display-block",
+                    onclick="expander('expand_all_failed', this)",
+                )
+
+            if formatter.additional_info:
+                with div(cls="feature-additional-info-container", id="additional-info"):
+                    # Generating Additional info results
+                    with div(cls="feature-additional-info"):
+                        for key, item in formatter.additional_info.items():
+                            div(
+                                f"{key}: {item}",
+                                cls=f"feature-additional-info-row {key.lower()}",
+                            )
 
         # Feature data container.
         with div(cls="feature-container", id=f"f{self.counter}"):
@@ -291,7 +296,12 @@ class Scenario:
         self.tags = [Tag(tag) for tag in scenario.effective_tags]
 
         self.location = scenario.location
-        self.status = Status.skipped.name
+
+        if self._scenario.status:
+            self.status = self._scenario.status
+        else:
+            self.status = Status.skipped.name
+
         self.duration = 0.0
         self.match_id = -1
         self.steps_finished = False
@@ -303,6 +313,7 @@ class Scenario:
 
         self.saved_matched_filename = None
         self.saved_matched_line = None
+
         # Process before_scenario errors.
         self.report_error(scenario)
 
@@ -477,15 +488,14 @@ class Scenario:
 
             # Simple container for name + duration.
             with div(cls="scenario-info"):
-
-                with div(
+                div(
+                    f"Scenario: {self.name}",
                     cls="scenario-name",
                     id=f"f{self.feature.counter}-s{self.counter}",
                     onclick="expand_this_only(this)",
-                ):
-                    span(f"Scenario: {self.name}")
-                with div(cls="scenario-duration"):
-                    span(f"Scenario duration: {self.duration:.2f}s")
+                )
+
+                div(f"Scenario duration: {self.duration:.2f}s", cls="scenario-duration")
 
         with div(
             cls=f"scenario-capsule {self.status}",
@@ -568,43 +578,39 @@ class Step:
             margin_top_cls = "margin-top"
 
         if self.commentary_override:
-            with div(cls=f"step-capsule commentary {margin_top_cls}"):
-                pre(f"{self.text}")
+            pre(f"{self.text}", cls=f"step-capsule commentary {margin_top_cls}")
+
         else:
             with div(cls=f"step-capsule {self.status} {margin_top_cls}"):
-                with div(cls="step-status"):
+                # Behave defined status strings are:
+                # "passed" "failed" "undefined" "skipped".
+                # Modify these values for high contrast usage.
+                # Step status for high contrast - "PASS" "FAIL" "SKIP".
+                high_contrast_status = {
+                    "passed": "PASS",
+                    "failed": "FAIL",
+                    "undefined": "SKIP",
+                    "skipped": "SKIP",
+                    "untested": "SKIP",
+                }
 
-                    # Behave defined status strings are:
-                    # "passed" "failed" "undefined" "skipped".
-                    # Modify these values for high contrast usage.
-                    high_contrast_status = {
-                        "passed": "PASS",
-                        "failed": "FAIL",
-                        "undefined": "SKIP",
-                        "skipped": "SKIP",
-                        "untested": "SKIP",
-                    }
-                    # Step status for high contrast - "PASS" "FAIL" "SKIP".
-                    span(high_contrast_status[self.status])
+                div(high_contrast_status[self.status], cls="step-status")
 
+                # Step decorator.
                 with div(cls="step-decorator"):
-                    # Step decorator.
-                    with b():
-                        i(self.keyword + " ")
+                    b(i(self.keyword + " "))
                     formatter.make_bold_text(self.name)
 
-                with div(cls="step-duration"):
-                    short_duration = f"{self.duration:.2f}s"
-                    # Step duration.
-                    span(f"({short_duration})")
+                # Step duration.
+                short_duration = f"{self.duration:.2f}s"
+                div(f"({short_duration})", cls="step-duration")
 
                 # Make the link only when the link is provided.
                 if self.location_link:
-                    with div(cls="link flex-left-space"):
-                        with a(href=self.location_link):
-                            span(self.location)
+                    a(self.location, cls="flex-left-space", href=self.location_link)
                 else:
                     span(self.location, cls="flex-left-space")
+
             # Still in non-commentary.
             self.generate_text()
             self.generate_table()
@@ -629,12 +635,12 @@ class Step:
         def _create_download_button():
             args = f"'embed_{embed_data.uid}','{use_caption}'"
             onclick = f"download_embed({args})"
-            a(
-                "[Download]",
-                href="#/",
-                cls="embed_download margin-bottom",
-                onclick=onclick,
-            )
+            with div():
+                span(
+                    "Download",
+                    cls="button margin-bottom",
+                    onclick=onclick,
+                )
 
         # Rule for embed_data.download_button as None - default value.
         if embed_data.download_button is None:
@@ -654,7 +660,46 @@ class Step:
             # Create download for all cases.
             _create_download_button()
 
-    def generate_embed(self, embed_data):  # pylint: disable=too-many-branches
+    def generate_embed_content(self, mime_type, data):
+        """
+        Generate content of the embed based on the mime_type.
+
+        :param mime_type: Mime type of the data to embed.
+        :type mime_type: str
+
+        :param data: Data to be embedded.
+        :type data: Unspecified.
+        """
+
+        # Actual Embed.
+        if "video/webm" in mime_type:
+            with video(
+                width="1024",
+                controls="",
+            ):
+                source(src=f"data:{mime_type};base64,{data}", type=mime_type)
+
+        if "image/png" in mime_type:
+            img(src=f"data:{mime_type};base64,{data}")
+
+        if "text" in mime_type:
+            if "html" in mime_type:
+                data_span = span()
+                data_span.add_raw_string(data)
+            elif "markdown" in mime_type:
+                data_md = markdown.markdown(data)
+                data_span = span()
+                data_span.add_raw_string(data_md)
+            else:
+                span(data)
+
+        if "link" in mime_type:
+            # expected format: set( [link, label], ... )
+            for single_link in data:
+                with div():
+                    a(single_link[1], href=single_link[0])
+
+    def generate_embed(self, embed_data):
         """
         Converts embed data into HTML.
 
@@ -662,6 +707,7 @@ class Step:
         returned to user for later modification of data, we want to
         prevent accidental call of this.
         """
+
         caption = embed_data.caption
         mime_type = embed_data.mime_type
         data = embed_data.data
@@ -689,56 +735,21 @@ class Step:
 
         with div(cls="messages"):
             with div(cls="embed-capsule"):
-
                 # Embed Caption.
-                with div(cls="embed_button collapse"):
-                    with div(cls="link"):
-                        # Label to be shown.
-                        embed_id = f"embed_{embed_data.uid}"
-                        with a(
-                            href="#/",
-                            onclick=f"collapsible_toggle('{embed_id}',this)",
-                        ):
-                            span(use_caption)
+                div(
+                    use_caption,
+                    cls="embed_button collapse",
+                    onclick=f"collapsible_toggle('embed_{embed_data.uid}',this)",
+                )
 
+                # Embed content.
                 with pre(
                     cls="embed_content",
                     id=f"embed_{embed_data.uid}",
                     style="display: none",
                 ):
-
                     self.generate_download_button(embed_data, data, use_caption)
-
-                    # Actual Embed.
-                    if "video/webm" in mime_type:
-                        with video(
-                            width="1024",
-                            controls="",
-                        ):
-                            source(
-                                src=f"data:{mime_type};base64,{data}", type=mime_type
-                            )
-
-                    if "image/png" in mime_type:
-                        img(src=f"data:{mime_type};base64,{data}")
-
-                    if "text" in mime_type:
-                        if "html" in mime_type:
-                            data_span = span()
-                            data_span.add_raw_string(data)
-                        elif "markdown" in mime_type:
-                            data_md = markdown.markdown(data)
-                            data_span = span()
-                            data_span.add_raw_string(data_md)
-                        else:
-                            span(data)
-
-                    if "link" in mime_type:
-                        # expected format: set( [link, label], ... )
-                        with div():
-                            for single_link in data:
-                                with a(href=single_link[0]):
-                                    span(single_link[1])
+                    self.generate_embed_content(mime_type, data)
 
     def generate_table(self):
         """
@@ -751,7 +762,6 @@ class Step:
 
         # Generate Table.
         with table(cls="table"):
-
             # Make a heading.
             with thead(
                 onclick="collapsible_toggle("
@@ -778,12 +788,12 @@ class Step:
             return
         with table(cls="table"):
             # Do not make the table header.
-            with thead(
-                onclick="collapsible_toggle("
-                f"'table_{PrettyHTMLFormatter.table_number}')"
-            ):
-                line = tr()
-                line += th("Text")
+            # with thead(
+            #     onclick="collapsible_toggle("
+            #     f"'table_{PrettyHTMLFormatter.table_number}')"
+            # ):
+            #     line = tr()
+            #     line += th("Text")
             # Make the body.
             with tbody(id=f"table_{PrettyHTMLFormatter.table_number}"):
                 # Make rows.
@@ -890,9 +900,7 @@ class Tag:
             # side for links to bugzilla. Tags come with structure
             # [<tag>, None] or [<tag>, <bugzilla_link/git_link>].
             if self.has_link():
-                with div(cls="link"):
-                    with a(href=self._link):
-                        span("@" + self.behave_tag)
+                a("@" + self.behave_tag, href=self._link)
             else:
                 span("@" + self.behave_tag)
 
@@ -921,6 +929,8 @@ class PrettyHTMLFormatter(Formatter):
 
         self.suite_start_time = datetime.now()
 
+        self._closed = False
+
         # Some type of icon can be set.
         self.icon = None
 
@@ -962,6 +972,8 @@ class PrettyHTMLFormatter(Formatter):
                 short_key = key.replace(additional_info_path, "")
                 self.additional_info[short_key] = item
 
+        atexit.register(self._force_close)
+
     def _str_to_bool(self, value):
         assert value.lower() in ["true", "false", "yes", "no", "0", "1"]
         return value.lower() in ["true", "yes", "1"]
@@ -1099,10 +1111,71 @@ class PrettyHTMLFormatter(Formatter):
         """
         self.icon = icon
 
+    def _add_unexecuted_scenario(self):
+        class DummyStep:  # pylint: disable=too-few-public-methods
+            """
+            Dummy step setting minimum attributes,
+            so formatter would not crash.
+            """
+
+            keyword = "Before"
+            name = "scenario"
+            location = None
+            duration = None
+            error_message = None
+            text = None
+            table = None
+            status = Status.failed.name
+
+        class DummyScenario:  # pylint: disable=too-few-public-methods
+            """
+            Dummy scenario setting minimum attributes,
+            so formatter would not crash.
+            """
+
+            name = "Unknown scenario"
+            effective_tags = []
+            description = ""
+            location = None
+            steps = [DummyStep]
+            error_message = None
+            status = Status.failed.name
+
+        feature = self.current_feature
+        background = feature._background  # pylint: disable=protected-access
+        feature.add_background(None)
+        pseudo_steps = self.pseudo_steps
+        self.pseudo_steps = False
+        self.before_scenario_finish(Status.failed.name)
+        self.scenario(DummyScenario)
+        feature.add_background(background)
+        self.pseudo_steps = pseudo_steps
+
+    def _force_close(self):
+        """
+        Called by `atexit`, set status of last step to failed and `close()`.
+        """
+        feature = self.current_feature
+        scenario = self.current_scenario
+        if not scenario and feature:
+            # Create unexecuted scenario, if there are unprocessed embeds.
+            if feature.to_embed:
+                self._add_unexecuted_scenario()
+        # refresh scenario
+        scenario = self.current_scenario
+        if scenario:
+            scenario.status = Status.failed.name
+            step = scenario.current_step
+            step.status = Status.failed.name
+        self.close()
+
     def close(self):
         """
         Generates the entire html page with dominate.
         """
+        if self._closed:
+            return
+        self._closed = True
         # Set finish time of the last feature.
         current_feature = self.current_feature
         if current_feature:
